/* ********************************** */
/* *                                * */
/* * Programme C                    * */
/* * ===========                    * */
/* *                                * */
/* * DLL de communication entre     * */
/* * scilab et la couche VISA       * */
/* *                                * */
/* *                                * */
/* * Derniere modif - Fevrier 2007  * */
/* ********************************** */


// Version 1.1 (Sept 2005): Gestion de l'interface VISA (incl. TCP IP)
// Version 1.2 (Fev 2007) : Lecture des binblocks flottants
// 1.4 (Mars 2007)

/* ********************************** */
/* Cration de la librairie           */
/* ---------------------------------- */

/**************************************/
/* Sous Windows :                     */
/*  
Localiser les fichiers visa.h et visa.lib
ils sont fournis avec NI-VISA et Agilent VISA pour les version Windows

Pour la cration de la DLL, sous Microsoft Visual C

* Project > Project Setting > Onglet "Link" > Object / library modules : AJOUTER VISA32.lib

* Tools > Option > Onglet Directories > Show directories for Include files : Mettre le repertoire ou se trouve VISA.h (ex: C:\Program Files\VISA\Winnt\include


* Tools > Option > Onglet Directories > Show directories for Librairy files : Mettre le repertoire ou se trouve VISA32.lib (ex: C:\Program Files\VISA\Winnt\lib\msc

/ *********************************** /


/**************************************/
/* Sous Linux :                     */
/*  
Localiser les fichiers visa.h et visa.lib
ils sont fournis avec NI-VISA 

Pour la cration de la librairie, avec GCC


* Comilation de la librairie
gcc -c -fPIC main.c

* Edition des liens
gcc -shared main.o /usr/local/vxipnp/linux/bin/libvisa.so -o Sci_HPIB.so

/ *********************************** /




/*  Pour Windows, remplacer cette ligne par */
#include <visa.h>

/* Pour Linux */
/*  #include "/usr/local/vxipnp/linux/include/visa.h" */


#include <stdio.h>
#include <string.h>
#include <stdlib.h>


//Session VISA par dfaut
static ViSession DefRM;
static long ltimout;

//Tableau des sessions gpib
#define NB_ADR_GPIB 32
struct visa_gpib_device_session
{
	ViSession vid;
	short initialized;
};


static struct visa_gpib_device_session t_session[NB_ADR_GPIB];

static struct visa_gpib_device_session asrl_session;

/* ====================
    Version 1.1
	-----------
	ADD ON : VISA direct (incl Socket Raw)
	*/

//Tableau des sessions visa 
#define NB_ADR_VISA 32
struct visa_device_session
{
	ViSession vid;
	char name[200];
	short initialized;
};
static struct visa_device_session v_session[NB_ADR_GPIB];

/* Cette variable ne sert que pour Write_sub et Read_sub (instrument  2 adresses) */
static ViSession my_instrument; 



/***************/
/*  GPIB_INIT  */
/***************/
void gpib_init(char *a)
{
	short i;

	viOpenDefaultRM(&DefRM);

	printf("\n*=====================*\n");
	printf("* Scilab GPIB Toolbox *\n");
	printf("*              v. 1.4 *\n");
	printf("*---------------------*\n");
	printf("*  Tibault Reveyrand  *\n");
	printf("*     CNES / IRCOM    *\n");
	printf("*=====================*\n");
	printf("\n");
	printf("http://membres.lycos.fr/treveyrand");
	printf("\n");

	/* Tableau des Sessions GPIB */
	for (i=0; i<NB_ADR_GPIB; i++)
	{
		t_session[i].initialized = 0;
		t_session[i].vid = VI_NULL;
	}

	/* 1 Session COM */
	asrl_session.initialized = 0;
	asrl_session.vid = VI_NULL;

	/* Tableau des Sessions VISA (autre que GPIB0:...) */
	for (i=0; i<NB_ADR_VISA; i++)
	{
		v_session[i].initialized = 0;
		v_session[i].vid = VI_NULL;
	}




//	atexit(gpib_close());
}


/*****************************/
/* Dialogue avec le port COM */
/*****************************/
// asrl1::
// Rajout - Aout 2005


void asrl_open()
{
	char m_adr[32];
	sprintf(m_adr,"ASRL1::INSTR");
	
	viOpen(DefRM,m_adr,VI_NULL,VI_NULL,&(asrl_session.vid));

	asrl_session.initialized = 1;
}


void asrl_write(int *num_commande, char *commande)
{
	int i,taille;

	if (!asrl_session.initialized) asrl_open();
	

	for (i=1;i<=*num_commande;i++)
		{
			taille=strlen(commande);
			viPrintf (asrl_session.vid, commande);
			commande=commande+taille+1;
		}
	

}

void asrl_read(int *num_commande, int *taille_sortie, char sortie[])
{
	// char buf [256] = {0};
	char buf [4096] = {0};
	int	i,j;
 	int	offset;

	if (!asrl_session.initialized) asrl_open();
	
	i=0;
	offset=0;
	for (j=1;j<=(*num_commande);j++)
	{
		
		viScanf (asrl_session.vid, "%t", &buf);
		// printf ("Instrument : %s\n", buf);
	
		i=0;
		while ((i<(*taille_sortie))&&(buf[i]!=0))
		{
			sortie[i+offset]=buf[i]; i++;
		}
	    sortie[i]=0;i++;
		offset=offset+strlen(sortie+offset)+1;
	
	}
}


void asrl_close()
{
	if (asrl_session.initialized)
			viClose(asrl_session.vid);
}

/* ****************************** */

void visa_open(int *VISA_ID, char *name)
{
	// ex : name = "TCPIP::10.0.0.2::23::SOCKET"
	char m_adr[100];
	sprintf(m_adr,name);
	
	viOpen(DefRM,m_adr,VI_NULL,VI_NULL,&(v_session[*VISA_ID].vid));

	v_session[*VISA_ID].initialized = 1;
}

void visa_close(char *a)
{
	short i;
	for (i=0; i<NB_ADR_VISA; i++)
	{
		if (v_session[i].initialized)
			viClose(v_session[i].vid);
	}
}

void visa_write (int *VISA_ID, char *commande)
{
	if (v_session[*VISA_ID].initialized) 
	{
		viPrintf (v_session[*VISA_ID].vid, commande);
	}
	
}

void visa_read (int *VISA_ID, int *taille_sortie, char *sortie)
{
	char buf [4096] = {0};
	int	i;

	if (v_session[*VISA_ID].initialized) 
	{
		viScanf (v_session[*VISA_ID].vid, "%t", &buf);
		i=0;
		while ((i<(*taille_sortie))&&(buf[i]!=0))
		{
			sortie[i]=buf[i]; i++;
		}
	    sortie[i]=0;i++;
	}
}


/* ****************************** */




/****************/
/*  GPIB_CLOSE  */
/****************/
void gpib_close(char *a)
{
	short i;
	for (i=0; i<NB_ADR_GPIB; i++)
	{
		if (t_session[i].initialized)
			viClose(t_session[i].vid);
	}
	asrl_close();
	viClose(DefRM);
}




void gpib_open_device(int GPIB_ID)
{
	char m_adr[32];
	sprintf(m_adr,"GPIB0::%u::INSTR",GPIB_ID);
	
	viOpen(DefRM,m_adr,VI_NULL,VI_NULL,&(t_session[GPIB_ID].vid));

	t_session[GPIB_ID].initialized = 1;
}



void gpib_set_timout(int *GPIB_ID, int *value_ms)
{

	if (!t_session[*GPIB_ID].initialized) gpib_open_device(*GPIB_ID);
	
	viSetAttribute(t_session[*GPIB_ID].vid,VI_ATTR_TMO_VALUE,*value_ms);
	return;
}


void GPIB_write (int *GPIB_ID, int *num_commande, char *commande)
{
	int i,taille;

	if (!t_session[*GPIB_ID].initialized) gpib_open_device(*GPIB_ID);
	

	for (i=1;i<=*num_commande;i++)
		{
			taille=strlen(commande);
			viPrintf (t_session[*GPIB_ID].vid, commande);
			commande=commande+taille+1;
		}
	
}


void GPIB_write_block (int *GPIB_ID, char *buffer, int *taille_buffer)	
{
	unsigned long	actual;

	if (!t_session[*GPIB_ID].initialized) gpib_open_device(*GPIB_ID);
	

	viWrite(t_session[*GPIB_ID].vid, (ViBuf) buffer, *taille_buffer, &actual);
}



// #### COPIE D'UN FICHIER BINAIRE VERS LE BUS GPIB
long  fsize(FILE* fd)
{
   long size;
   fseek(fd, 0, SEEK_END);       /* aller en fin */
   size = ftell(fd);             /* lire la taille */
   return size;
}

void GPIB_write_binary_file (int *GPIB_ID, char *fichier)	
{
	unsigned long	actual;

	FILE * pFile;
	long lSize;
	char * buffer;
	size_t result;

	pFile = fopen ( fichier , "rb" );
	if (pFile!=NULL) 
	{

		// obtain file size:
		fseek (pFile , 0 , SEEK_END);
		lSize = ftell (pFile);
		rewind (pFile);

		// allocate memory to contain the whole file:
		buffer = (char*) malloc (sizeof(char)*lSize);
		if (buffer != NULL) 
		{

			// copy the file into the buffer:
			result = fread (buffer,1,lSize,pFile);
			if (result == lSize) 
			{

				/* the whole file is now loaded in the memory buffer. */
				if (!t_session[*GPIB_ID].initialized) gpib_open_device(*GPIB_ID);
				viWrite(t_session[*GPIB_ID].vid, (ViBuf) buffer, lSize, &actual);
			}
		}
	}

  // terminate
  fclose (pFile);
  free (buffer);

}



void GPIB_read (int *GPIB_ID, int *num_commande, int *taille_sortie, char sortie[])
{
	// char buf [256] = {0};
	char buf [4096] = {0};
	int	i,j;
 	int	offset;

	if (!t_session[*GPIB_ID].initialized) gpib_open_device(*GPIB_ID);
	
	i=0;
	offset=0;
	for (j=1;j<=(*num_commande);j++)
	{
		
		viScanf (t_session[*GPIB_ID].vid, "%t", &buf);
		// printf ("Instrument : %s\n", buf);
	
		i=0;
		while ((i<(*taille_sortie))&&(buf[i]!=0))
		{
			sortie[i+offset]=buf[i]; i++;
		}
	    sortie[i]=0;i++;
		offset=offset+strlen(sortie+offset)+1;
	
	}
}



void GPIB_read_data (int *GPIB_ID, int *taille_sortie, char sortie[])
{ 
	char buf [23*3201] ={0};
	int	i;
 	if (!t_session[*GPIB_ID].initialized) gpib_open_device(*GPIB_ID);
	
	viScanf (t_session[*GPIB_ID].vid, "%t", &buf);
	i=0;
	while ((i<(*taille_sortie))&&(buf[i]!=0))
	{
	  sortie[i]=buf[i]; i++;
	}
}


void GPIB_read_stb (int *GPIB_ID, unsigned short *result)
{   
	if (!t_session[*GPIB_ID].initialized) gpib_open_device(*GPIB_ID);
    viReadSTB (t_session[*GPIB_ID].vid, result);
}








void AWG2021_save_signal (int *GPIB_ID,int *taille, double *data, char *name)
{
	/* Version Simple : pas de gestion des markers */

	int	taille_buffer,offset,i,j;
	char *buffer;
	char size_data[10];

	short tampon;
	int nb_command;

	nb_command=1;

// Preparation de l'envoi
	taille_buffer=16+strlen(name);
	buffer= (char*) malloc (taille_buffer);
	sprintf(buffer,":DATA:DEST ");
	buffer[11]=34;
	sprintf(buffer+12,"%s",name);
	buffer[12+strlen(name)]=34;
	buffer[12+strlen(name)+1]=10;
	buffer[12+strlen(name)+2]=13;
	buffer[12+strlen(name)+3]=0;
	// printf("%s",buffer);
	GPIB_write(GPIB_ID,&nb_command,buffer);
	free(buffer);

// Construction du binblock  partir d'un tableau de double normalis
	taille_buffer=2*(*taille);
	sprintf(size_data,"%d",taille_buffer);
	// printf("taille buffer %d",taille_buffer);
	taille_buffer=7+2+taille_buffer+strlen(size_data);
	// 7 pour ":CURVE "
	// 2 pour "#x"
		
	buffer= (char*) malloc (taille_buffer);
	sprintf(buffer,":CURVE #");
	sprintf(buffer+8,"%d",strlen(size_data));
	sprintf(buffer+9,"%s",size_data);
	offset=9+strlen(size_data);

	for (i=0;i<*taille;i++) {
		tampon=(short)(((((double) data[i])+1)/2)*4095);
		for (j=0;j<2;j++) 
			buffer[offset+i*2+1-j]=*( ((char *)&tampon)+j);
	}
	
// Debugage
//	for (i=0;i<taille_buffer;i++) printf("%c",buffer[i]);

// Envoi via GPIB
	GPIB_write_block (GPIB_ID, buffer, &taille_buffer);

  
	free(buffer);


}


void SMU200A_save_signal (int *GPIB_ID,int *taille, double *data, char *name, double *clock)
{
	/* {TYPE: SMU-WV,0} {CLOCK: 10e6} {WAVEFORM-401: #

	*/

//	char entete[50];
//	sprintf(entete,"{TYPE: SMU-WV,0} {CLOCK: %E} {WAVEFORM-",*clock);
//	taille_buffer=2*(*taille);



	/* Version Simple : pas de gestion des markers */

	int	taille_buffer,offset,i,j;
	char *buffer;
	char size_data[10];

	short tampon;
	int nb_command;

	nb_command=1;

// Preparation de l'envoi
	taille_buffer=16+strlen(name);
	buffer= (char*) malloc (taille_buffer);
	sprintf(buffer,":DATA:DEST ");
	buffer[11]=34;
	sprintf(buffer+12,"%s",name);
	buffer[12+strlen(name)]=34;
	buffer[12+strlen(name)+1]=10;
	buffer[12+strlen(name)+2]=13;
	buffer[12+strlen(name)+3]=0;
	// printf("%s",buffer);
	GPIB_write(GPIB_ID,&nb_command,buffer);
	free(buffer);

// Construction du binblock  partir d'un tableau de double normalis
	taille_buffer=2*(*taille);
	sprintf(size_data,"%d",taille_buffer);
	// printf("taille buffer %d",taille_buffer);
	taille_buffer=7+2+taille_buffer+strlen(size_data);
	// 7 pour ":CURVE "
	// 2 pour "#x"
		
	buffer= (char*) malloc (taille_buffer);
	sprintf(buffer,":CURVE #");
	sprintf(buffer+8,"%d",strlen(size_data));
	sprintf(buffer+9,"%s",size_data);
	offset=9+strlen(size_data);

	for (i=0;i<*taille;i++) {
		tampon=(short)(((((double) data[i])+1)/2)*4095);
		for (j=0;j<2;j++) 
			buffer[offset+i*2+1-j]=*( ((char *)&tampon)+j);
	}
	
// Debugage
//	for (i=0;i<taille_buffer;i++) printf("%c",buffer[i]);

// Envoi via GPIB
	GPIB_write_block (GPIB_ID, buffer, &taille_buffer);

  
	free(buffer);


	

}




void AWG520_save_signal (int *GPIB_ID,int *taille, double *data, char *name, double *clock)
{
	/* Version Simple : pas de gestion des markers */

	int	taille_buffer,offset,i,j;
	char *buffer;
	char size_data[10];
	char size_data_2[10];
	char clock_line[50];
	float tampon;


// Preparation de l'envoi
	sprintf(clock_line,"CLOCK %E\r\n",*clock);

	// printf("\n %s \n",clock_line);
	taille_buffer=5*(*taille);
	sprintf(size_data,"%d",taille_buffer);
	i=12+strlen(size_data)+taille_buffer+strlen(clock_line)+1;
	sprintf(size_data_2,"%d",i);
	
	taille_buffer=i+2+strlen(size_data_2)+14+strlen(name);
	
	buffer= (char*) malloc (taille_buffer);
	
	offset=12+strlen(name);

	sprintf(buffer,":MMEM:DATA ");
	buffer[11]=34;					/* " */
	sprintf(buffer+12,"%s",name);	/* nom de fichier */
	buffer[offset]=34;				/* " */
	buffer[offset+1]=44;			/* , */
	buffer[offset+2]=35;			/* # */
	sprintf(buffer+offset+3,"%d",strlen(size_data_2));		/* index */
	sprintf(buffer+offset+4,"%s",size_data_2 );				/* taille en byte */
	
	offset=offset+4+strlen(size_data_2);
	sprintf(buffer+offset,"MAGIC 1000%c%c%c",13,10,35);
	offset=offset+13;

	sprintf(buffer+offset,"%d",strlen(size_data));			/* index */
	sprintf(buffer+offset+1,"%s",size_data );				/* taille en byte */
	
	offset=offset+strlen(size_data)+1;
	
	for (i=0;i<(*taille);i++)
	{
		tampon=(float) data[i];
		for (j=0;j<4;j++) 
			buffer[offset+i*5+j]=*( ((char *)&tampon)+j);
		buffer[offset+i*5+4]=0;
	}
	buffer[offset+4]=1;

	offset=offset+(*taille)*5;

	sprintf(buffer+offset,"%s",clock_line );

	
// Debugage
//	printf("\r\n Taille = %d \r\n",taille_buffer);
//	for (i=0;i<taille_buffer;i++) printf("%c",buffer[i]);
//	printf("\r\n");
//	for (i=0;i<taille_buffer;i++) printf("<%d>",buffer[i]);


// Envoi via GPIB

	GPIB_write_block (GPIB_ID, buffer, &taille_buffer);

  
	free(buffer);


}


void GPIB_read_block_short (int *GPIB_ID,int *taille, double *data)
{
	unsigned long	actual;

	int	taille_buffer,i,offset;
	unsigned char *buffer;
	char size_data[10];
	short	tampon=0;

	if (!t_session[*GPIB_ID].initialized) gpib_open_device(*GPIB_ID);

	sprintf(size_data,"%d",((*taille)*2));
	taille_buffer=((*taille)*2)+2+strlen(size_data)+1;	/* +1 : Octet EOL du binblock	*/
	buffer= (char*) malloc (taille_buffer);
	viRead(t_session[*GPIB_ID].vid, (ViBuf) buffer, taille_buffer, &actual);
	offset=2+strlen(size_data);
	
	// for (i=0;i<taille_buffer;i++) printf("<%c : %d> \r\n",buffer[i],buffer[i]);
	
	for (i=0;i<(*taille);i++)							/* Probleme d'ecriture de short sur des adresse impaires */
	{													/* On passe donc par une variable tampon dclare en Short */

	*((char *)(&tampon))=buffer[offset+(i*2)+1];	/* Inversion Manuelle de l'ordre des byte dans le short */
	*(((char *)(&tampon))+1)=buffer[offset+(i*2)];
	// printf ("%d",tampon);
	data[i]=(double) tampon;						/* Conversion en double pour Scilab */
	}


	free(buffer);
}



void GPIB_read_block_byte (int *GPIB_ID,int *taille, double *data)
{
	unsigned long	actual;

	int	taille_buffer,i,offset;
	char *buffer;
	char size_data[10];


	if (!t_session[*GPIB_ID].initialized) gpib_open_device(*GPIB_ID);

	sprintf(size_data,"%d",((*taille)));
	taille_buffer=((*taille))+2+strlen(size_data)+1;	/* +1 : Octet EOL du binblock	*/
	buffer= (char*) malloc (taille_buffer);
	viRead(t_session[*GPIB_ID].vid, (ViBuf) buffer, taille_buffer, &actual);
	offset=2+strlen(size_data);
	
	for (i=0;i<(*taille);i++)							/* Probleme d'ecriture de short sur des adresse impaires */
		data[i]=(double) buffer[i+offset];						/* Conversion en double pour Scilab */
	
	free(buffer);
}


void GPIB_read_block_float (int *GPIB_ID,int *taille, double *data)
{
	unsigned long	actual;

	int	taille_buffer,i,offset;
	char *buffer;
	char size_data[10];
	float	tampon=0;


	if (!t_session[*GPIB_ID].initialized) gpib_open_device(*GPIB_ID);

	sprintf(size_data,"%d",((*taille)*4));
	taille_buffer=((*taille)*4)+2+strlen(size_data)+1;	/* +1 : Octet EOL du binblock	*/
	buffer= (char*) malloc (taille_buffer);
	viRead(t_session[*GPIB_ID].vid, (ViBuf) buffer, taille_buffer, &actual);
	offset=2+strlen(size_data);
	
	for (i=0;i<(*taille);i++)	/* Probleme d'ecriture de short sur des adresse impaires */
	{
		*(((char *)(&tampon)))=buffer[offset+(i*4)+3];	/* Inversion Manuelle de l'ordre des byte dans le short */
		*(((char *)(&tampon))+1)=buffer[offset+(i*4)+2];
		*(((char *)(&tampon))+2)=buffer[offset+(i*4)+1];
		*(((char *)(&tampon))+3)=buffer[offset+(i*4)];
		
		data[i]=(double) tampon;
	}

	free(buffer);
}


void GPIB_read_block_floatNI (int *GPIB_ID,int *taille, double *data)
{
	unsigned long	actual;

	int	taille_buffer,i,offset;
	char *buffer;
	char size_data[10];
	float	tampon=0;


	if (!t_session[*GPIB_ID].initialized) gpib_open_device(*GPIB_ID);

	sprintf(size_data,"%d",((*taille)*4));
	taille_buffer=((*taille)*4)+2+strlen(size_data)+1;	/* +1 : Octet EOL du binblock	*/
	buffer= (char*) malloc (taille_buffer);
	viRead(t_session[*GPIB_ID].vid, (ViBuf) buffer, taille_buffer, &actual);
	offset=2+strlen(size_data);
	
	for (i=0;i<(*taille);i++)	/* Probleme d'ecriture de short sur des adresse impaires */
	{
		*(((char *)(&tampon)))=buffer[offset+(i*4)];	/* Inversion Manuelle de l'ordre des byte dans le short */
		*(((char *)(&tampon))+1)=buffer[offset+(i*4)+1];
		*(((char *)(&tampon))+2)=buffer[offset+(i*4)+2];
		*(((char *)(&tampon))+3)=buffer[offset+(i*4)+3];
		
		data[i]=(double) tampon;
	}

	free(buffer);
}



void GPIB_read_block_double (int *GPIB_ID,int *taille, double *data)
{
	unsigned long	actual;

	int	taille_buffer,i,offset;
	char *buffer;
	char size_data[10];
	double	tampon=0;


	if (!t_session[*GPIB_ID].initialized) gpib_open_device(*GPIB_ID);

	sprintf(size_data,"%d",((*taille)*8));
	taille_buffer=((*taille)*8)+2+strlen(size_data)+1;	/* +1 : Octet EOL du binblock	*/
	buffer= (char*) malloc (taille_buffer);
	viRead(t_session[*GPIB_ID].vid, (ViBuf) buffer, taille_buffer, &actual);
	offset=2+strlen(size_data);
	
	for (i=0;i<(*taille);i++)	/* Probleme d'ecriture de short sur des adresse impaires */
	{
		*(((char *)(&tampon)))=buffer[offset+(i*8)+7];
		*(((char *)(&tampon))+1)=buffer[offset+(i*8)+6];
		*(((char *)(&tampon))+2)=buffer[offset+(i*8)+5];
		*(((char *)(&tampon))+3)=buffer[offset+(i*8)+4];
		*(((char *)(&tampon))+4)=buffer[offset+(i*8)+3];
		*(((char *)(&tampon))+5)=buffer[offset+(i*8)+2];
		*(((char *)(&tampon))+6)=buffer[offset+(i*8)+1];
		*(((char *)(&tampon))+7)=buffer[offset+(i*8)];
		
		data[i]= tampon;
	}

	free(buffer);
}


/* Ouverure et fermeture de la session dans l'instruction */
/* Sert au sous adresses GPIB de l'alim AGILENT 63000A */

void GPIB_write_sub (int *GPIB_ID, int *GPIB_SUB_ID, int *num_commande, char *commande)
{
	int i,taille;
	char m_adr[32];
/*	ViSession my_instrument; */

	sprintf(m_adr,"GPIB0::%u::%u::INSTR",*GPIB_ID,*GPIB_SUB_ID);
	/* printf(m_adr); */

	

	viOpen(DefRM,m_adr,VI_NULL,VI_NULL,&(my_instrument));


	for (i=1;i<=*num_commande;i++)
		{
			taille=strlen(commande);
			viPrintf (my_instrument, commande);
			/* printf(commande); */
			commande=commande+taille+1;
		}

  
	viClose(my_instrument);


}

/* Ouverure et fermeture de la session INSTRUMENT (pas VISA) dans l'instruction mme */
/* Tout ca a cause de ces putain de sous adresses GPIB de l'alim Agilent 63000A */


void GPIB_read_sub (int *GPIB_ID, int *GPIB_SUB_ID, int *num_commande, int *taille_sortie, char sortie[])
{
	char buf [256] = {0};
	int	i,j;
 	int	offset;
	char m_adr[32];
	ViSession my_instrument;


	sprintf(m_adr,"GPIB0::%u::%u::INSTR",*GPIB_ID,*GPIB_SUB_ID);
	
	viOpen(DefRM,m_adr,VI_NULL,VI_NULL,&(my_instrument));
	
	i=0;
	offset=0;
	for (j=1;j<=(*num_commande);j++)
	{
		
		viScanf (my_instrument, "%t", &buf);
	/* 	printf ("Instrument : %s\n", buf); */
	
		i=0;
		while ((i<(*taille_sortie))&&(buf[i]!=0))
		{
			sortie[i+offset]=buf[i]; i++;
		}
	    sortie[i]=0;i++;
		offset=offset+strlen(sortie+offset)+1;
	
	}
	viClose(my_instrument);
}


